/* 
** Module   : ImageConfig.java
** Abstract : stores IMAGE specific state
**
** Copyright (c) 2014-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20140924 Created initial version.
** 002 EVL 20141208 Adding attributes for image loading and color conversion.
** 003 EVL 20150202 Adding image offset related members.
** 004 EVL 20150216 Adding TRANSPARENT, STRETCH-TO-FIT and RETAIN-SHAPE options support.
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * Contains IMAGE specific widget state.
 */
public class ImageConfig
extends BaseConfig
{
   /** Image file name. */
   public String image = null;

   /** Image CONVERT-3D-COLORS attribute flag. */
   public boolean convert3D = false;
   
   /** Image TRANSPARENT attribute flag. */
   public boolean transparent = false;
   
   /** Image STRETCH-TO-FIT attribute flag. */
   public boolean stretchToFit = false;
   
   /** Image RETAIN-SHAPE attribute flag. */
   public boolean retainShape = false;
   
   /** X offset in pixel units inside the image data to start drawing image. */
   public int offsetX = -1;
   
   /** Y offset in pixel units inside the image data to start drawing image. */
   public int offsetY = -1;
   
   /** Row offset in character units inside the image data to start drawing image. */
   public double offsetRow = 0.0;
   
   /** Column offset in character units inside the image data to start drawing image. */
   public double offsetCol = 0.0;
   
   /**
    * Default constructor (only used in deserialization).
    */
   public ImageConfig()
   {
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected ImageConfig(int id)
   {
      super(id);
   }

   /**
    * Create a new config and associate it with the given widget.
    * 
    * @param    id
    *           The ID of the widget to which this configuration belongs.
    */
   protected ImageConfig(WidgetId id)
   {
      super(id);
   }
   /**
    * Set new values from the provided configuration instance.
    * 
    * @param    config
    *           The instance from which to take values.
    */
   @Override
   public void applyConfig(WidgetConfig config)
   {
      super.applyConfig(config);
      
      ImageConfig cfg = (ImageConfig) config;
      
      image        = cfg.image;
      convert3D    = cfg.convert3D;
      transparent  = cfg.transparent;
      stretchToFit = cfg.stretchToFit;
      retainShape  = cfg.retainShape;
      offsetX      = cfg.offsetX;
      offsetY      = cfg.offsetY;
      offsetRow    = cfg.offsetRow;
      offsetCol    = cfg.offsetCol;
   }

   /**
    * Replacement for the default object reading method. The latest
    * state is read from the input source.  The parent's state is
    * read first.
    * 
    * @param    in
    *           The input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   @Override
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      super.readExternal(in);
      convert3D    = in.readBoolean();
      transparent  = in.readBoolean();
      stretchToFit = in.readBoolean();
      retainShape  = in.readBoolean();
      offsetX      = in.readInt();
      offsetY      = in.readInt();
      offsetRow    = in.readDouble();
      offsetCol    = in.readDouble();
      image        = (String) in.readObject();
   }

   /**
    * Replacement for the default object writing method. The latest
    * state is written to the output destination.  The parent's state is
    * written first.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   @Override
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      super.writeExternal(out);
      out.writeBoolean(convert3D);
      out.writeBoolean(transparent);
      out.writeBoolean(stretchToFit);
      out.writeBoolean(retainShape);
      out.writeInt(offsetX);
      out.writeInt(offsetY);
      out.writeDouble(offsetRow);
      out.writeDouble(offsetCol);
      out.writeObject(image);
   }
}
